@@ -11,66 +11,20 @@ import subprocess
 
 import model
 
-# linux only ?
-from ptrace.debugger.debugger import PtraceDebugger
-# ptrace fork
-from memory_mapping import readProcessMappings, MemoryDumpMemoryMapping
+from memory_mapper import MemoryMapper
 
 log=logging.getLogger('haystack')
 
 environSep = ':'
 
 
+
 class StructFinder:
   ''' Generic tructure mapping '''
-  def __init__(self, pid=None, mmap=False, memdump=None):
-    if pid is None and memdump is None:
-      raise ValueError('You have to choose pid or dumpfile')
-    if not (pid is None) and not (memdump is None):
-      raise ValueError('You have to choose pid or dumpfile')
-    if not (pid is None):
-      self.initPid(pid,mmap)
-    elif not (memdump is None):
-      self.initMemdump(memdump)
-
-  def initMemdump(self,memdump):
-    mem = MemoryDumpMemoryMapping(memdump, 0, os.fstat(memdump.fileno()).st_size) ## is that valid ?
-    self.mappings=[mem]
-    log.debug('memdump initialised %s'%(self.mappings[0]))
-  
-  def initPid(self,pid, mmap):
-    self.dbg = PtraceDebugger()
-    self.process = self.dbg.addProcess(pid,is_attached=False)
-    if self.process is None:
-      log.error("Error initializing Process debugging for %d"% pid)
-      raise IOError
-      # ptrace exception is raised before that
-    tmp = readProcessMappings(self.process)
-    self.mappings=[]
-    remains=[]
-    t0=time.time()
-    for m in tmp :
-      if mmap:
-        ### mmap memory in local space
-        m.mmap()
-        #log.warning('mmap() : %d'%(len(m.local_mmap)))
-      if ( m.pathname == '[heap]' or 
-           m.pathname == '[vdso]' or
-           m.pathname == '[stack]' or
-           m.pathname is None ):
-        self.mappings.append(m)
-        continue
-      remains.append(m)
-    #tmp = [x for x in remains if not x.pathname.startswith('/')] # delete memmapped dll
-    tmp=remains
-    tmp.sort(key=lambda x: x.start )
-    tmp.reverse()
-    self.mappings.extend(tmp)
-    self.mappings.reverse()
-    if mmap:
-      ### mmap done, we can release process...
-      self.process.cont()
-      log.info('Memory mmaped, process released after %02.02f secs'%(time.time()-t0))
+  def __init__(self, mappings):
+    self.mappings = mappings
+    log.debug('StructFinder on %s'%(self.mappings[0]) )
+    return
 
   def find_struct(self, struct, hintOffset=0, maxNum = 10, maxDepth=10 , fullScan=False):
     if not fullScan:
@@ -188,35 +142,32 @@ def checkModulePath(typ):
     sys.path.append( moddir )
   return
 
-def findStruct(pid, struct, maxNum=1, fullScan=False, nommap=False):
+def _findStruct(pid=None, memfile=None, memdump=None, struct=None, maxNum=1, 
+              fullScan=False, nommap=False, hint=None, debug=None ):
   ''' '''
   if type(struct) != type(ctypes.Structure):
     raise TypeError('struct arg must be a ctypes.Structure')
   checkModulePath(struct) # add to sys.path
   struct = '.'.join([struct.__module__,struct.__name__]) # we pass a str anyway...
-  cmd_line=[sys.executable, getMainFile(), "%s"%struct, "--pid", "%d"%pid, 'search',  '--maxnum', str(int(maxNum))] #, '--nommap'
-  if fullScan:
-    cmd_line.append('--fullscan')
+  cmd_line=[sys.executable, getMainFile(), "%s"%struct]
+  if debug:
+    cmd_line.insert(2,"--debug")
   if nommap:
-    cmd_line.append('--nommap')
-  outs=_callFinder(cmd_line)
-  if len(outs) == 0:
-    log.error("The %s has not been found."%(struct))
-    return None
-  #
-  return outs
-
-def findStructInFile(filename, struct, hint=None, maxNum=1, fullScan=False):
-  ''' '''
-  if type(struct) != type(ctypes.Structure):
-    raise TypeError('struct arg must be a ctypes.Structure')
-  checkModulePath(struct) # add to sys.path
-  struct = '.'.join([struct.__module__,struct.__name__])
-  cmd_line=[sys.executable, getMainFile(), "--debug", "%s"%struct, "--fromdump", filename, 'search',  '--maxnum', str(int(maxNum))] #, '--nommap'
+    cmd_line.insert(2,'--nommap')
+  # three cases  
+  if pid:       
+    ### live PID. with mmap or not 
+    cmd_line.extend(["--pid", "%d"%pid ])
+  elif memfile: 
+    ### proc mappings dump file
+    cmd_line.extend([ "--memfile", memfile] )
+  # always add search
+  cmd_line.extend(['search',  '--maxnum', str(int(maxNum))] )
   if fullScan:
     cmd_line.append('--fullscan')
   if hint:
     cmd_line.extend(['--hint', str(hex(hint))])
+  # call me
   outs=_callFinder(cmd_line)
   if len(outs) == 0:
     log.error("The %s has not been found."%(struct))
@@ -224,13 +175,33 @@ def findStructInFile(filename, struct, hint=None, maxNum=1, fullScan=False):
   #
   return outs
 
-def refreshStruct(pid, struct, offset):
+def findStruct(pid, struct, maxNum=1, fullScan=False, nommap=False, debug=False):
+  return _findStruct(pid=pid, struct=struct, maxNum=maxNum, fullScan=fullScan, nommap=nommap, debug=debug)
+  
+def findStructInFile(filename, struct, hint=None, maxNum=1, fullScan=False, debug=False):
+  return _findStruct(memfile=filename, struct=struct, maxNum=maxNum, fullScan=fullScan, debug=debug)
+
+
+def refreshStruct(pid, struct, offset, debug=False, nommap=False):
   ''' '''
   if type(struct) != type(ctypes.Structure):
     raise TypeError('struct arg must be a ctypes.Structure')
   checkModulePath(struct) # add to sys.path
   struct = '.'.join([struct.__module__,struct.__name__])
-  cmd_line=[sys.executable, getMainFile(),  '%s'%struct, '--pid', "%d"%pid ,'refresh',  "0x%lx"%offset ]
+  cmd_line=[sys.executable, getMainFile(),  '%s'%struct]
+  if debug:
+    cmd_line.insert(2,"--debug")
+  if nommap:
+    cmd_line.insert(2,'--nommap')
+  # three cases  
+  if pid:       
+    ### live PID. with mmap or not 
+    cmd_line.extend(["--pid", "%d"%pid ])
+  elif memfile: 
+    ### proc mappings dump file
+    cmd_line.extend([ "--memfile", memfile] )
+  # always add search
+  cmd_line.extend(['refresh',  "0x%lx"%offset] )
   instance,validated=_callFinder(cmd_line)
   if not validated:
     log.error("The session_state has not been re-validated. You should look for it again.")
@@ -241,24 +212,24 @@ def refreshStruct(pid, struct, offset):
 def usage(parser):
   parser.print_help()
   sys.exit(-1)
-
-
+  
 def argparser():
   rootparser = argparse.ArgumentParser(prog='StructFinder', description='Parse memory structs and pickle them.')
   rootparser.add_argument('--string', dest='human', action='store_const', const=True, help='Print results as human readable string')
   rootparser.add_argument('--debug', dest='debug', action='store_const', const=True, help='setLevel to DEBUG')
+  rootparser.add_argument('--nommap', dest='mmap', action='store_const', const=False, default=True, help='disable mmap()-ing')
   rootparser.add_argument('structType', type=str, help='Structure type name')
   
   target = rootparser.add_mutually_exclusive_group(required=True)
   target.add_argument('--pid', type=int, help='Target PID')
-  target.add_argument('--fromdump', type=argparse.FileType('r'), dest='memdump', action='store', default=None, help='Use a memory dump instead of a live process ID')
-  
+  target.add_argument('--memfile', type=argparse.FileType('r'), dest='memfile', action='store', default=None, 
+                                    help='Use a memory dump instead of a live process ID')
+    
   subparsers = rootparser.add_subparsers(help='sub-command help')
   search_parser = subparsers.add_parser('search', help='search help')
   search_parser.add_argument('--fullscan', action='store_const', const=True, default=False, help='do a full memory scan, otherwise, restrict to the heap')
   search_parser.add_argument('--maxnum', type=int, action='store', default=1, help='Limit to maxnum numbers of results')
   search_parser.add_argument('--hint', type=int, action='store', default=1, help='hintOffset to start at')
-  search_parser.add_argument('--nommap', action='store_const', const=True, default=False, help='disable mmap()-ing')
   search_parser.set_defaults(func=search)
   #
   refresh_parser = subparsers.add_parser('refresh', help='refresh help')
@@ -276,12 +247,11 @@ def getKlass(name):
 
 
 def search(args):
+  log.debug('args: %s'%args)
   structType=getKlass(args.structType)
-  if args.pid :
-    finder = StructFinder(pid=args.pid, mmap=(not args.nommap))
-  else : # from dump
-    finder = StructFinder(memdump=args.memdump)
-    log.debug('starting a memory file dump search')
+  mappings = MemoryMapper(args).getMappings()
+  finder = StructFinder(mappings)
+
   outs=finder.find_struct( structType, hintOffset=args.hint ,maxNum=args.maxnum, fullScan=args.fullscan)
   #return
   if args.human:
@@ -293,21 +263,20 @@ def search(args):
   else:
     ret=[ (ss.toPyObject(),addr) for ss, addr in outs]
     log.info("%s %s"%(ret[0], type(ret[0]) )   )
-    #model.findCtypesInPyObj(ret)
+    if model.findCtypesInPyObj(ret):
+      log.error('=========************======= CTYPES STILL IN pyOBJ !!!! ')
     print pickle.dumps(ret)
   return outs
 
 
 def refresh(args):
+  log.debug(args)
 
   addr=int(args.addr,16)
   structType=getKlass(args.structType)
 
-  if args.pid :
-    finder = StructFinder(pid=args.pid, mmap=(not args.nommap))
-  else : # from dump
-    finder = StructFinder(memdump=args.memdump)
-    log.debug('starting a memory file dump search')
+  mappings = MemoryMapper(args).getMappings()
+  finder = StructFinder(mappings)
   
   memoryMap = model.is_valid_address_value(addr, finder.mappings)
   if not memoryMap:
@@ -320,6 +289,8 @@ def refresh(args):
        print '( %s, %s )'%(instance.toString(),validated)
     else:
       d=(instance.toPyObject(),validated)
+      if model.findCtypesInPyObj(d[0]):
+        log.error('=========************======= CTYPES STILL IN pyOBJ !!!! ')
       print pickle.dumps(d)
   else:
     if args.human:
